@@ -29,7 +29,7 @@ import os
 import time
 
 
-__VERSION__ = 0.30
+__VERSION__ = 0.31
 
 
 # kernel definitions for ioctl commands
@@ -107,11 +107,14 @@ _N800_RADIO_DEV = "/dev/radio0"
 _N900_RADIO_DEV = "/dev/radio1"
 _N800_MIXER_DEV = "/dev/mixer"
 
+# sysfs interface path on the N900
+_N900_RADIO_SYSFS = "/sys/class/i2c-adapter/i2c-3/3-0022"
+
 # signal scanning parameters
 _SIGNAL_LOCK_TIME = 0.1
 _SIGNAL_TRIES = 1
 _SIGNAL_SAMPLE_SLEEP = 0.005
-_SIGNAL_THRESHOLD = 20
+_SIGNAL_THRESHOLD = 5
 _SCAN_STEP_KHZ = 50
 
 
@@ -202,14 +205,16 @@ class FMRadio(object):
         """
 
         try:
-            self.cancel_scanning()        
+            self.cancel_scanning()
             self.set_mute(True)
             if (self.__device == "RX-34"):
                 self.set_frequency(0)
-            os.close(self.__fd)
         except:
-            import traceback; traceback.print_exc()
+            #import traceback; traceback.print_exc()
+            pass
                 
+        os.close(self.__fd)
+
         
     def set_fm_band(self, band):
         """
@@ -356,7 +361,31 @@ class FMRadio(object):
         
         inp = struct.pack("Ll", _V4L2_CID_AUDIO_MUTE, value and 1 or 0)
         ioctl(self.__fd, _VIDIOC_S_CTRL, inp)
+
+
+    def get_rds(self):
+        """
+        Returns the RDS strings on the N900, or simply empty strings on the
+        N800.
+        The fields are:
+         * PI (Program Identification): unique station identifier
+         * PS (Program Service): eight-character string containing station name
+         * RT (Radio Text): 64-character string containing arbitrary content
         
+        @return: tuple of strings (PI, PS, RT)
+        """
+
+        if (self.__device == "RX-51"):
+            pi = open(_N900_RADIO_SYSFS + "/rds_pi", "r").read()
+            ps = open(_N900_RADIO_SYSFS + "/rds_ps", "r").read()
+            rt = open(_N900_RADIO_SYSFS + "/rds_rt", "r").read()
+        else:
+            pi = ""
+            ps = ""
+            rt = ""
+
+        return (pi, ps, rt)
+
         
     def get_signal_strength(self):
         """
