@@ -44,10 +44,12 @@ class FMScale(Widget):
         Widget.__init__(self)
         
         # kinetic scrolling
-        kscr = KineticScroller(self)
-        kscr.connect_clicked(self.__on_click)
-        kscr.connect_scrolling_stopped(self.__on_stop_scrolling)
+        self.__kscr = KineticScroller(self)
+        self.__kscr.set_drag_threshold(0)
+        #kscr.connect_clicked(self.__on_click)
+        self.__kscr.connect_scrolling_stopped(self.__on_stop_scrolling)
 
+        self.set_size(100, 100)
 
 
     def connect_frequency_changed(self, cb, *args):
@@ -60,19 +62,21 @@ class FMScale(Widget):
         self._connect(self.EVENT_SCANNING_ACTIVATED, cb, *args)
 
 
+    """
     def __on_click(self, px, py):
     
         w, h = self.get_size()
         v = self.get_value()
         
-        if (px < 80):
-            self.emit_event(self.EVENT_SCANNING_ACTIVATED, -1)
-        elif (px > w - 80):
-            self.emit_event(self.EVENT_SCANNING_ACTIVATED, 1)
-        elif (px < w / 2):
+        #if (px < 80):
+        #    self.emit_event(self.EVENT_SCANNING_ACTIVATED, -1)
+        #elif (px > w - 80):
+        #    self.emit_event(self.EVENT_SCANNING_ACTIVATED, 1)
+        if (px < w / 2):
             self.step_down()
         else:
             self.step_up()
+    """
 
 
     def __on_stop_scrolling(self):
@@ -100,13 +104,19 @@ class FMScale(Widget):
     def set_size(self, w, h):
     
         old_w, old_h = self.get_size()
-        Widget.set_size(self, w, h)
-        
+
         if ((w, h) != (old_w, old_h)):
+            freq = self.get_value()
+            Widget.set_size(self, w, h)
             self.__prepare_scale()
             self.__buffer = Pixmap(None, w, h)
             self.__buffer_invalidated = True
-              
+            self.set_value(freq)
+            self.__kscr.set_drag_area(80, 0, w - 160, h)
+
+        else:
+            Widget.set_size(self, w, h)
+        
         
     def render_this(self):
 
@@ -120,13 +130,14 @@ class FMScale(Widget):
 
         # render overlays
         TEMPORARY_PIXMAP.copy_buffer(self.__buffer, 0, 0, 0, 0, w, h)        
+        self.render_overlays(TEMPORARY_PIXMAP)
         #TEMPORARY_PIXMAP.fill_area(w / 2 - 5, 0, 10, h, "#aaaaffa0")
         
-        TEMPORARY_PIXMAP.draw_pixbuf(theme.mb_btn_previous_1,
-                                     0, (h - 64) / 2)
+        #TEMPORARY_PIXMAP.draw_pixbuf(theme.mb_btn_previous_1,
+        #                             0, (h - 64) / 2)
 
-        TEMPORARY_PIXMAP.draw_pixbuf(theme.mb_btn_next_1,
-                                     w - 64, (h - 64) / 2)
+        #TEMPORARY_PIXMAP.draw_pixbuf(theme.mb_btn_next_1,
+        #                             w - 64, (h - 64) / 2)
                                      
         TEMPORARY_PIXMAP.fill_area((w - 4) / 2, 0, 4, h,
                                    theme.color_fmradio_scale_needle)
@@ -278,7 +289,7 @@ class FMScale(Widget):
         """
     
         w, h = self.get_size()
-        return self.get_value_at(w / 2)        
+        return self.get_value_at(w / 2)
 
 
     def step_down(self):
@@ -344,4 +355,5 @@ class FMScale(Widget):
         r1, r2 = self.__range
         new_pos = int((freq - r1) * self.__scaling)
         self.animate(100, fx, [self.__offset, new_pos])
+        self.__offset = new_pos
 
