@@ -3,9 +3,11 @@ def get_classes():
     from HeadphonesMonitor import HeadphonesMonitor
     from SoundVolumeMonitor import SoundVolumeMonitor
     from PhoneMonitor import PhoneMonitor
+    from PolicyMonitor import PolicyMonitor
     return [HeadphonesMonitor,
             SoundVolumeMonitor,
-            PhoneMonitor]
+            PhoneMonitor,
+            PolicyMonitor]
 
 
 
